<?php

namespace FiloBlu\Rma\Plugin\Model;

use Exception;
use FiloBlu\Rma\Model\Rma\Status\History;
use FiloBlu\Rma\Model\Rma\Status\HistoryFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Rma\Model\Rma\Source\Status;
use Magento\Store\Model\ScopeInterface;
use Psr\Log\LoggerInterface;
use Magento\Rma\Model\Item\Attribute\Source\Status as ItemStatus;
use FiloBlu\Rma\Helper\SerializerHelper;
use Magento\Rma\Api\RmaRepositoryInterface;
use FiloBlu\Rma\Api\Model\Config\Reader;
use FiloBlu\Rma\Api\RmaExtensionAttributesRepositoryInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use FiloBlu\Rma\Model\RmaEmailManagment;

/**
 * Class Rma
 * @package FiloBlu\Rma\Plugin\Model
 */
class Rma
{
    /**
     * @var string
     */
    const XML_PATH_RECEIVED_EMAIL_ENABLE = 'sales_email/magento_rma_receive/enabled';

    /**
     * @var string
     */
    const XML_PATH_RMA_REASON_MAPPING = 'sales/rma_reason/reason';

    /**
     * @var HistoryFactory
     */
    protected $historyFactory;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var SerializerHelper
     */
    protected $serializer;
    /**
     * @var RmaExtensionAttributesRepositoryInterface
     */
    private $rmaExtensionAttributesRepository;
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;
    /**
     * @var ChangeSizeAttributeHelper
     */
    private $changeSizeAttributeHelper;

    /**
     * @var RmaEmailManagment
     */
    protected $rmaEmailManagment;


    /**
     * Rma constructor.
     * @param HistoryFactory $historyFactory
     * @param ScopeConfigInterface $scopeConfig
     * @param RmaRepositoryInterface $rmaRepository
     * @param Reader $reader
     * @param LoggerInterface $logger
     * @param SerializerHelper $serializer
     * @param RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository
     * @param OrderRepositoryInterface $orderRepository
     * @param ChangeSizeAttributeHelper $changeSizeAttributeHelper
     * @param RmaEmailManagment $rmaEmailManagment
     */
    public function __construct(
        HistoryFactory $historyFactory,
        ScopeConfigInterface $scopeConfig,
        RmaRepositoryInterface $rmaRepository,
        Reader $reader,
        LoggerInterface $logger,
        SerializerHelper $serializer,
        RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository,
        OrderRepositoryInterface $orderRepository,
        ChangeSizeAttributeHelper $changeSizeAttributeHelper,
        RmaEmailManagment $rmaEmailManagment)
    {
        $this->historyFactory = $historyFactory;
        $this->scopeConfig = $scopeConfig;
        $this->rmaRepository = $rmaRepository;
        $this->reader = $reader;
        $this->logger = $logger;
        $this->serializer = $serializer;
        $this->rmaExtensionAttributesRepository = $rmaExtensionAttributesRepository;
        $this->orderRepository = $orderRepository;
        $this->changeSizeAttributeHelper = $changeSizeAttributeHelper;
        $this->rmaEmailManagment = $rmaEmailManagment;
    }

    /***
     * @param \Magento\Rma\Model\Rma $subject
     * @param $result
     * @return \Magento\Rma\Model\Rma
     */
    public function afterSaveRma(\Magento\Rma\Model\Rma $subject, $result)
    {
        if ($result === false) {
            return $subject;
        }

        try {
            $this->shouldUpdateStatus($subject, $result);
            //manage RmaEmail
            $this->rmaEmailManagment->checkIfSendRmaEmail($subject);

        } catch (Exception $e) {
            $this->logger->critical($e);
            return $result;
        }

        return $result;
    }


    /**
     * @param \Magento\Rma\Model\Rma $subject
     * @param $result
     */
    public function shouldUpdateStatus(\Magento\Rma\Model\Rma $subject, $result){
        
        switch (true) {

            case ($result->getStatus() == Status::STATE_PROCESSED_CLOSED || $result->getStatus() == Status::STATE_DENIED || $result->getStatus() == Status::STATE_CLOSED) :
                foreach ($subject->getItems() as $item){

                    $rma_item_id = $item->getEntityId();
                    $rmaId = $subject->getEntityId();
                    $changeSizeOrderId = $this->rmaExtensionAttributesRepository->getChangeSizeOrderId($rma_item_id);
                    if($changeSizeOrderId) {
                        $changeSizeOrder = $this->orderRepository->get($changeSizeOrderId);
                        $payment = $changeSizeOrder->getPayment()->getMethod();
                        if ($payment == \FiloBlu\Rma\Model\Payment\ChangeSizePay::PAYMENT_CODE) {
                            if ($item->getResolution() == $this->changeSizeAttributeHelper->getChangeSizeAttributeOptionId() and $item->getStatus() == Status::STATE_APPROVED) {
                                $changeSizeOrder = $this->orderRepository->get($changeSizeOrderId);

                                $changeSizeOrder->setStatus('payment_confirmed')->setState('processing');
                            }else {
                                $changeSizeOrder = $this->orderRepository->get($changeSizeOrderId);
                                $changeSizeOrder->setStatus('canceled')->setState('canceled');
                            }
                            $changeSizeOrder->addStatusHistoryComment("Status and State updated programatically by rma with id $rmaId update");
                            $changeSizeOrder->save();
                        }
                    }
                }
                break;

            case  ($result->getStatus() == Status::STATE_PENDING) :

                /** @var $result \Magento\Rma\Model\Rma */
                $items = $result->getItems();
                $standard_reasons_config = $this->reader->read();
                $automatic_approval = array();
                foreach($standard_reasons_config as $reason):
                    if($reason['automatic_approval']==1):
                        $automatic_approval[] = $reason['id'];
                    endif;
                endforeach;
                /** @var \Magento\Rma\Api\Data\ItemInterface $item */
                $auth = 0;

                $itemsComments ="";
                foreach($items as $item):
                    $item_status = $item->getStatus();
                    $item_brand_reason = $item->getReason();
                    $mapping = $this->serializer->unserialize($this->getConfigValue(self::XML_PATH_RMA_REASON_MAPPING, $subject->getStoreId()));
                    foreach ($mapping as $reason):
                        if($item_brand_reason == $reason['brand_reason'] && in_array($reason['standard_reason'],$automatic_approval)):
                            // avanzamento dello stato dell'item
                            $item->setStatus(ItemStatus::STATE_AUTHORIZED);
                            $itemsComments  .=" -- ".$item->getProductAdminName()." automatically authorized -- ";
                            $item->setQtyAuthorized($item->getQtyRequested());
                            $item->save();
                            $auth++;
                            break;
                        endif;
                    endforeach;
                endforeach;

                if($auth > 0 && $auth==count($items)):
                    $this->rmaRepository->get($result->getEntityId())
                        ->setStatus(Status::STATE_AUTHORIZED)
                        ->save();
                elseif($auth > 0):
                    $this->rmaRepository->get($result->getEntityId())
                        ->setStatus(Status::STATE_PARTIAL_AUTHORIZED)
                        ->save();
                endif;

                if($auth > 0){
                    $statusHistory = $this->historyFactory->create();
                    $statusHistory->setRmaEntityId($result->getEntityId());
                    $statusHistory->saveComment($itemsComments,false,true);
                }

                break;
        }

    }




    /**
     * Return store configuration value
     *
     * @param string $path
     * @param int $storeId
     * @return mixed
     */
    protected function getConfigValue($path, $storeId)
    {
        return $this->scopeConfig->getValue(
            $path,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

}

